package main

import (
	"flag"
	"fmt"
	"log"
	"net"
	"net/http"
	"os"

	"example/hello/api"
	"example/hello/db"

	"github.com/go-chi/chi/v5"
	"github.com/go-chi/chi/v5/middleware"
	m "github.com/oapi-codegen/nethttp-middleware"
)

func main() {
	port := flag.String("port", "3000", "Port for test HTTP server")
	flag.Parse()

	DB := db.ConnectDB()
	db.Migrate(DB)

	swagger, err := api.GetSwagger()
	if err != nil {
		fmt.Fprintf(os.Stderr, "Error loading swagger spec\n: %s", err)
		os.Exit(1)
	}
	swagger.Servers = nil

	r := chi.NewRouter()
	r.Use(m.OapiRequestValidator(swagger))
	r.Use(middleware.Logger)

	petStore := &api.PetStore{DB: DB}
	api.HandlerFromMux(petStore, r)

	s := &http.Server{
		Handler: r,
		Addr:    net.JoinHostPort("0.0.0.0", *port),
	}

	// And we serve HTTP until the world ends.
	log.Fatal(s.ListenAndServe())
}
